package ru.ispras.kmb.vedomost;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Ведомость учета посещения.
 * 
 * @author demakov
 */
public class VedomostN2D implements Vedomost
{
    // String name
    // int day
    
    // List<T>
    // Set<T>
    // Map<K,V>
    
    Map<String,Set<Integer>> name2day = new HashMap<>();
    
    /* (non-Javadoc)
     * @see ru.ispras.kmb.vedomost.Vedomost#add(int, java.lang.String)
     */
    @Override
    public void add( int day, String name ) throws IllegalDayException 
    {
        if( !(1 <= day && day <= 31) )
        {
            throw new IllegalDayException( "day = " + day );
        }
        Set<Integer> days = name2day.get( name );
        if( days == null )
        {
            days = new HashSet<>();
            name2day.put( name, days);
        }
        days.add( day );
    }
    
    /* (non-Javadoc)
     * @see ru.ispras.kmb.vedomost.Vedomost#get(java.lang.String)
     */
    @Override
    public  List<Integer> get( String name ) 
    { 
        Set<Integer> days = name2day.get( name );
        if( days == null ) days = new HashSet<>();
        name2day.put( name, days );
        return Collections.unmodifiableList( new ArrayList<>( days ) ); 
    }
    
    /* (non-Javadoc)
     * @see ru.ispras.kmb.vedomost.Vedomost#get(int)
     */
    @Override
    public  Set<String> get( int day ) 
    { 
        Set<String> names = new HashSet<>();
        for( String name: name2day.keySet() )
        {
            Set<Integer> days = name2day.get( name );
            if( days.contains( day ) )
            {
                names.add( name );
            }
        }
        return names;
    }
    
    public static void main( String[] args ) throws IllegalDayException
    {
        Vedomost v = new VedomostN2D();
        v.add( 1, "Ivanov" );
        v.add( 3, "Ivanov" );
        v.add( 4, "Ivanov" );
        v.add( 6, "Ivanov" );
        v.add( 8, "Ivanov" );
        v.add( 2, "Petrov" );
        v.add( 3, "Petrov" );
        v.add( 6, "Petrov" );

        printVedomost( v );  
        
        List<Integer> days = v.get( "Ivanov" );
        
        System.out.print( "Ivanov" );
        for( int i = 0; i < days.size(); i++ )
        {
            System.out.print( " " + days.get( i ) );
        }
        System.out.println();
    }

    private static void printVedomost( Vedomost v )
    {
        for( int day = 1; day <= 31; day++ )
        {
            Set<String> names = v.get( day );
        
            for( String name: names )
            {
                System.out.println( day + " " + name );
            }
        }
    }
}
