package ru.ispras.kmb.vedomost;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.junit.Test;

public abstract class VedomostTest
{
    public static final String SIDOROV = "Sidorov";

    public static final String PETROV = "Petrov";

    public static final String IVANOV = "Ivanov";
    
    protected Vedomost v;

    @Test
    public void sizeTest()
    {
        List<Byte> list = new ArrayList<Byte>();
        for( int i = 0; i < 20; i++ )
        {
            list.add((byte)i);
        }
    }
    
    @Test
    public void testEmptyGetString()
    {
        List<Integer> days = v.get( IVANOV );
        
        assertEquals( "not empty list", 0, days.size() );
    }

    @Test
    public void testEmptyGetInt()
    {
        Set<String> names = v.get( 8 );
        
        assertEquals( "not empty set", 0, names.size() );
    }
    
    @Test
    public void testGetSameString()
    {
        try
        {
            v.add( 1, IVANOV );
            v.add( 1, IVANOV );
            v.add( 1, IVANOV );
            v.add( 1, IVANOV );
        }
        catch( IllegalDayException e )
        {
             System.out.println( e.toString() );
        }
        finally
        {
             
        }
        List<Integer> days = v.get( IVANOV );
        
        assertEquals( "list size not equal 1", 1, days.size() );
        assertEquals( "day is not 1", 1, (int)days.get(0) );
    }

    @Test
    public void testGetOtherString() throws IllegalDayException
    {
        v.add( 1, IVANOV );
        List<Integer> days = v.get( PETROV );
        
        assertEquals( "not empty list", 0, days.size() );
    }

    @Test
    public void testGetSameInt() throws IllegalDayException
    {
        v.add( 1, IVANOV );
        Set<String> names = v.get( 1 );
        
        assertEquals( "wrong set size", 1, names.size() );
        assertTrue( "name not in set", names.contains( IVANOV ) );
    }

    @Test
    public void testAddDayTwice() throws IllegalDayException
    {
        v.add( 1, IVANOV );
        v.add( 3, IVANOV );
        v.add( 1, PETROV );
        v.add( 5, SIDOROV );
        Set<String> names = v.get( 1 );
        
        assertEquals( "wrong set size", 2, names.size() );
        assertTrue( "name not in set", names.contains( IVANOV ) );
        assertTrue( "name not in set", names.contains( PETROV ) );
    }

    @Test    
    public void testAddNameTwice() throws IllegalDayException
    {
        v.add( 3, IVANOV );
        v.add( 1, IVANOV );
        List<Integer> days = v.get( IVANOV );
        
        assertEquals( "wrong list size", 2, days.size() );
        assertTrue( "wrong day", days.contains( 1 ) );
        assertTrue( "wrong day", days.contains( 3 ) );
    }
    
    @Test
    public void testIllegalDay() throws IllegalDayException
    {
        try
        {
            v.add( 0, IVANOV );
            fail( "no MyException" );
        }
        catch( IllegalDayException e )
        {
            assertEquals( "day = 0", e.getMessage() );
        }
    }
}
