package ru.ispras.kmb;

import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

public class IdSplitterTest
{
    protected IdSplitter splitter;
    
    @Before
    public void init()
    {
        splitter = new MyIdSplitter();    
    }
    
    @Test
    public void selfTest()
    {
        String[] res = splitter.split( "IdSplitter" );
        assertEquals( "two words", 2, res.length );
        assertEquals( "first", "id", res[0] );
        assertEquals( "second", "splitter", res[1] );
    }
    
    @Test
    public void nullTest()
    {
        String[] res = splitter.split( null );
        assertNotEquals( "not null", null, res );
        assertEquals( "empty", 0, res.length );
    }
    
    @Test
    public void emptyTest()
    {
        String[] res = splitter.split( "" );
        assertNotEquals( "not null", null, res );
        assertEquals( "empty", 0, res.length );
    }

    @Test
    public void oneLowerCharTest()
    {
        String[] res = splitter.split( "a" );
        assertEquals( "one word", 1, res.length );
        assertEquals( "first", "a", res[0] );
    }

    @Test
    public void oneUpperCharTest()
    {
        String[] res = splitter.split( "A" );
        assertEquals( "one word", 1, res.length );
        assertEquals( "first", "a", res[0] );
    }
    
    @Test
    public void oneUnderscoreTest()
    {
        String[] res = splitter.split( "_" );
        assertEquals( "empty", 0, res.length );
    }
    
    @Test
    public void oneWordTest()
    {
        String[] res = splitter.split( "abc" );
        assertEquals( "one word", 1, res.length );
        assertEquals( "first", "abc", res[0] );
    }
    
    @Test
    public void twoWordsCamelTest()
    {
        String[] res = splitter.split( "twoWords" );
        assertEquals( "two words", 2, res.length );
        assertEquals( "first", "two", res[0] );
        assertEquals( "second", "words", res[1] );
    }
    
    @Test
    public void twoWordsUnderscoreTest()
    {
        String[] res = splitter.split( "two_words" );
//        assertEquals( "" );
        assertEquals( "two words", 2, res.length );
        assertEquals( "first", "two", res[0] );
        assertEquals( "second", "words", res[1] );
    }
    
    @Test
    public void twoWordsSpaceTest()
    {
        String[] res = splitter.split( "two words" );
        assertEquals( "two words", 2, res.length );
        assertEquals( "first", "two", res[0] );
        assertEquals( "second", "words", res[1] );
    }
    
    private void printStringArray( String[] res )
    {
        for( String id: res )
        {
            System.out.println( id );
        }
    }
}
